#if defined _weapon_limit_included
	#endinput
#endif
#define _weapon_limit_included

#include <cstrike>

enum WL_Event
{
	WL_OnBuyLimitedWeapon,
	WL_OnLimitChanged
};

typeset WL_CB
{
	/**
	 *	WL_OnBuyLimitedWeapon
	 *	Игрок пытается купить ограниченное оружие.
	 *
	 *	@note					Если разрешить покупку, то оружие упадёт на землю, т.к. всё ещё ограничено.
	 *							Предотвратить  падение можно, выдав игроку иммунитет. Но в некоторых случаях и он не поможет (см. ConVar wl_immunity_when_zero ).
	 *
	 *	@param client			Индекс игрока. Возможно бот. Возможно есть иммунитет, который не сработал из-за: @param limit 0 + ConVar wl_immunity_when_zero 0.
	 *	@param sWeapon			Имя оружия без weapon_. Для ножей это всегда "knife". Регистр может быть как и "awp", так и "AwP".
	 *	@param limit			Ограничено 'x' на команду. 0 = полностью запрещено для команды, но всё ещё разрешено игрокам с иммунитетом, если ConVar wl_immunity_when_zero 1.
	 *	@param warningSound		Проигрывать звук предупреждения, если покупка запрещена? По умолчанию true.
	 *
	 *	@return					true = разрешить, false = запретить.
	 */
	function bool (int client, const char[] sWeapon, int limit, bool &warningSound);
	
	/**
	 *	WL_OnLimitChanged
	 *	Лимит оружия изменён (с old_ на new_) как минимум для одной команды.
	 *
	 *	@note					Если old_ и new_ совпадают, то лимит для команды не был изменён. 
	 *	tt_limit/ct_limit		-1 = не ограничено. 0 = запрещено всей команде. 1 = одно оружие на команду, и т.д. Значение от -1 до MaxClients.
	 */
	function void (CSWeaponID weapon_id, int old_tt_limit, int new_tt_limit, int old_ct_limit, int new_ct_limit);
}

native void WL_HookEvent  (WL_Event event, WL_CB callback);
native void WL_UnhookEvent(WL_Event event, WL_CB callback);

/////

/**
 *	Узнать ограничено ли оружие.
 *
 *	@param weapon_id		id оружия.
 *	@param tt_limit			Лимит для команды  T. -1 = не ограничено. 0 = запрещено всей команде. 1 = одно оружие на команду, и т.д.
 *	@param ct_limit			Лимит для команды CT.
 *	tt_ratio/ct_ratio		Если не 0, то работает по принципу "одно оружие на 'x' товарищей по команде".
 *
 *	@return					true  = оружие ограничено минимум для одной команды.
 *							false = оружие не ограничено.
 */
native bool WL_GetWeaponLimit(CSWeaponID weapon_id, int &tt_limit, int &ct_limit, int &tt_ratio=0, int &ct_ratio=0);

/**
 *	Ограничить оружие или снять ограничение.
 *
 *	@param weapon_id		id оружия. CSWeapon_NONE = всё оружие.
 *	@param tt_limit			Лимит для команды  T. -1 = не ограничено. 0 = запрещено всей команде. 1 = одно оружие на команду, и т.д. -2 = оставить без изменений.
 *	@param ct_limit			Лимит для команды CT.
 *	@param ratio			Если true, то ограничение работает так: "одно оружие на 'x' товарищей по команде". Значение должно быть от 2 и выше. Подробнее в weapon_limit.txt.
 */
native void WL_SetWeaponLimit(CSWeaponID weapon_id, int tt_limit=-2, int ct_limit=-2, bool ratio=false);

/////

/**
 *	Есть ли у игрока иммунитет, позволяющий использовать ограниченное оружие.
 */
native bool WL_HasImmunity(int client);

/**
 *	Выдать/ отобрать иммунитет, позволяющий использовать ограниченное оружие.
 *
 *	@note					Используйте в OnClientPostAdminCheck или позже.
 *							После OnClientDisconnect/OnMapStart иммунитет игрока очищается.
 *							Нет конфликта с другими плагинами:
 *							Когда plugin выдаёт иммунитет, то иммунитет плюсуется к существующему.
 *							Когда plugin отбирает иммунитет/выгружается, то отбирается именно тот иммунитет, который plugin выдал (отобрать чужой нельзя).
 *
 *	@param client			Индекс игрока. Может быть бот.
 *	@param immunity			true = выдать иммунитет, false = отобрать.
 */
native void WL_SetImmunity(int client, bool immunity);

/////

enum WL_ConVar
{
	WL_ENABLED,
	WL_COUNT_IMMUNE_PLAYERS,
	WL_IMMUNITY_WHEN_ZERO,
	WL_DROP_LIMITED_AT_ROUND_START
}

/**
 *	Возвращает ConVar значение, записанное в глобальной переменной.
 */
native any WL_GetConVarValue(WL_ConVar convar);

/////

/**
 *	Будет ли работать иммунитет при использовании указанного оружия.
 *
 *	@note					Если отключить, то игроки с иммунитетом будут считаться обычными игроками, т.е. иммунитет не будет работать.
 *							По умолчанию иммунитет работает (включен) для всего оружия.
 *							В OnMapStart включается для всего оружия.
 *							Как и в WL_SetImmunity, нет конфликта с другими плагинами .
 *
 *	@param weapon_id		id оружия. CSWeapon_NONE = всё оружие.
 *	@param immunity			true = включить. false = отключить (иммунитет игроков не работает).
 *	@param team				0 = для всех команд, 2 = T, 3 = CT.
 */
native void WL_SetWeaponImmunity(CSWeaponID weapon_id, bool immunity, int team=0);

/**
 *	Работает ли иммунитет при использовании указанного оружия?
 *
 *	@param tt_immunity		true = иммунитет работает для команды TT, false = не работает.
 *	@param ct_immunity
 *
 *	@return					true = информация получена. false = неверный/не поддерживается @param weapon_id.
 */
native bool WL_GetWeaponImmunity(CSWeaponID weapon_id, bool &tt_immunity, bool &ct_immunity);

/////

/**
 *	Получить ref список конкретного оружия, которое есть на карте.
 *
 *	@note					Никакого поиска не происходит, т.к. список уже готов.
 *							Используйте EntRefToEntIndex, чтобы убедиться, что оружие существует.
 *
 *	@param weapon_id		id оружия.
 *	@param arrayLength		Кол-во оружия в списке (ArrayList размер).
 *
 *	@return					ArrayList клон, который вы должны закрыть. Если null, то на карте нет такого оружия.
 */
native ArrayList WL_GetWeaponRefList(CSWeaponID weapon_id, int &arrayLength=0);

/**
 *	@return					true  = id поддерживается в текущей игре. Если это нож, то @param weapon_id всегда CSWeapon_KNIFE.
 *							false = игра/Weapon Limit не поддерживает это оружие.
 */
native bool WL_IsValidWeaponID(CSWeaponID &weapon_id);

/**
 *	@return					true  = оружие на земле и игрок может его подобрать.
 *							false = оружие не существует/не на земле/игрок мёртв/игрок не может подобрать оружие из-за ограничений.
 */
native bool WL_IsClientCanPickupWeapon(int client, int weapon_index);

public SharedPlugin __pl_WeaponLimitVar = 
{
	name = "WeaponLimitLib",	/* Имя библиотеки */
	file = "ws_vip/weapon_limit.smx",
	required = 0				/* Не требуется, т.к. включается с задержкой (позже обычных .smx). */
};

public void __pl_WeaponLimitVar_SetNTVOptional()
{
	MarkNativeAsOptional("WL_HookEvent");
	MarkNativeAsOptional("WL_UnhookEvent");
	
	MarkNativeAsOptional("WL_GetWeaponLimit");
	MarkNativeAsOptional("WL_SetWeaponLimit");
	
	MarkNativeAsOptional("WL_HasImmunity");
	MarkNativeAsOptional("WL_SetImmunity");
	
	MarkNativeAsOptional("WL_GetConVarValue");
	
	MarkNativeAsOptional("WL_SetWeaponImmunity");
	MarkNativeAsOptional("WL_GetWeaponImmunity");
	
	MarkNativeAsOptional("WL_GetWeaponRefList");
	MarkNativeAsOptional("WL_IsValidWeaponID");
	MarkNativeAsOptional("WL_IsClientCanPickupWeapon");
}

/* 

/////

// https://world-source.ru/hueco_mundo/weapon_limit_config.zip
#include <weapon_limit>

static const char	  g_sWeaponLimitLib[] = "WeaponLimitLib";
bool	g_bWeaponLimitLib = false;

public void OnAllPluginsLoaded() {
	if (LibraryExists(g_sWeaponLimitLib)) {
		OnWeaponLimitLibAdded();
	}
}
public void OnLibraryAdded	(const char[] name) {
	if (!strcmp(name, g_sWeaponLimitLib)) {
		OnWeaponLimitLibAdded();
	}
}
public void OnLibraryRemoved(const char[] name) {
	if (!strcmp(name, g_sWeaponLimitLib)) {
		g_bWeaponLimitLib = false;
	}
}
static void OnWeaponLimitLibAdded() {
	if(!g_bWeaponLimitLib) {
		g_bWeaponLimitLib = true;
		
		WL_HookEvent(..
	}
}

/////

*/